import logging
import os

from createrepo import yumbased
import rpmUtils
from pulp.plugins.util import verification


_LOGGER = logging.getLogger(__name__)


def get_package_xml(pkg_path, sumtype=verification.TYPE_SHA256):
    """
    Method to generate repo xmls - primary, filelists and other
    for a given rpm.

    :param pkg_path: package path on the filesystem
    :type  pkg_path: str

    :param sumtype: The type of checksum to use for creating the package xml
    :type  sumtype: basestring

    :return:    rpm metadata dictionary or empty if rpm path doesnt exist
    :rtype:     dict
    """
    ts = rpmUtils.transaction.initReadOnlyTransaction()
    try:
        # createrepo raises an exception if sumtype is unicode
        # https://bugzilla.redhat.com/show_bug.cgi?id=1290021
        sumtype_as_str = str(sumtype)
        po = yumbased.CreateRepoPackage(ts, pkg_path, sumtype=sumtype_as_str)
    except Exception, e:
        # I hate this, but yum doesn't use reasonable exceptions like IOError
        # and ValueError.
        _LOGGER.error(str(e))
        return {}
    # RHEL6 createrepo throws a ValueError if _cachedir is not set
    po._cachedir = None
    primary_xml_snippet = po.xml_dump_primary_metadata()
    primary_xml_snippet = primary_xml_snippet.decode('utf-8', 'replace')
    primary_xml_snippet = change_location_tag(primary_xml_snippet, pkg_path)
    metadata = {
        'primary': primary_xml_snippet.encode('utf-8'),
        'filelists': po.xml_dump_filelists_metadata(),
        'other': po.xml_dump_other_metadata(),
    }
    return metadata


def change_location_tag(primary_xml_snippet, relpath):
    """
    Transform the <location> tag to strip out leading directories so it
    puts all rpms in same the directory as 'repodata'

    :param primary_xml_snippet: snippet of primary xml text for a single package
    :type  primary_xml_snippet: unicode

    :param relpath: Package's 'relativepath'
    :type  relpath: unicode
    """

    basename = os.path.basename(relpath)
    start_index = primary_xml_snippet.find("<location ")
    end_index = primary_xml_snippet.find("/>", start_index) + 2  # adjust to end of closing tag

    first_portion = primary_xml_snippet[:start_index]
    end_portion = primary_xml_snippet[end_index:]
    location = """<location href="%s"/>""" % (basename)
    return first_portion + location + end_portion
